using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Fidelity.Components;
using CtrlLib;
using WealthLab;
using WealthLab.PosSizers;
using WealthLab.Indicators;

namespace WealthLab.PosSizers.Mockasino
{
    public class MockasinoSettings : UserControl
    {
        private IContainer components = null;
        private Label lblLong;
        private Label lblShort;
        private NumEdit numShort;
        private Label lblRSI;
        private NumEdit numRSI;
        private NumEdit numLong;

        public MockasinoSettings()
        {
            this.InitializeComponent();
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing && (this.components != null))
            {
                this.components.Dispose();
            }
            base.Dispose(disposing);
        }

        private void InitializeComponent()
        {
            this.numLong = new CtrlLib.NumEdit();
            this.lblLong = new System.Windows.Forms.Label();
            this.lblShort = new System.Windows.Forms.Label();
            this.numShort = new CtrlLib.NumEdit();
            this.lblRSI = new System.Windows.Forms.Label();
            this.numRSI = new CtrlLib.NumEdit();
            this.SuspendLayout();
            // 
            // numLong
            // 
            this.numLong.InputType = CtrlLib.NumEdit.NumEditType.SmallInteger;
            this.numLong.Location = new System.Drawing.Point(157, 58);
            this.numLong.MaxLength = 6;
            this.numLong.Name = "numLong";
            this.numLong.Size = new System.Drawing.Size(100, 20);
            this.numLong.TabIndex = 1;
            this.numLong.Text = "2";
            // 
            // lblLong
            // 
            this.lblLong.Location = new System.Drawing.Point(9, 58);
            this.lblLong.Name = "lblLong";
            this.lblLong.Size = new System.Drawing.Size(142, 20);
            this.lblLong.TabIndex = 31;
            this.lblLong.Text = "Max long pos";
            // 
            // lblShort
            // 
            this.lblShort.Location = new System.Drawing.Point(9, 88);
            this.lblShort.Name = "lblShort";
            this.lblShort.Size = new System.Drawing.Size(129, 20);
            this.lblShort.TabIndex = 33;
            this.lblShort.Text = "Max short pos";
            // 
            // numShort
            // 
            this.numShort.InputType = CtrlLib.NumEdit.NumEditType.SmallInteger;
            this.numShort.Location = new System.Drawing.Point(158, 88);
            this.numShort.MaxLength = 6;
            this.numShort.Name = "numShort";
            this.numShort.Size = new System.Drawing.Size(100, 20);
            this.numShort.TabIndex = 2;
            this.numShort.Text = "1";
            // 
            // lblRSI
            // 
            this.lblRSI.Location = new System.Drawing.Point(9, 26);
            this.lblRSI.Name = "lblRSI";
            this.lblRSI.Size = new System.Drawing.Size(129, 20);
            this.lblRSI.TabIndex = 35;
            this.lblRSI.Text = "RSI threshold";
            // 
            // numRSI
            // 
            this.numRSI.InputType = CtrlLib.NumEdit.NumEditType.Double;
            this.numRSI.Location = new System.Drawing.Point(157, 26);
            this.numRSI.MaxLength = 10;
            this.numRSI.Name = "numRSI";
            this.numRSI.Size = new System.Drawing.Size(100, 20);
            this.numRSI.TabIndex = 0;
            this.numRSI.Text = "40";
            // 
            // MockasinoSettings
            // 
            this.BackColor = System.Drawing.SystemColors.Control;
            this.Controls.Add(this.lblRSI);
            this.Controls.Add(this.numRSI);
            this.Controls.Add(this.lblShort);
            this.Controls.Add(this.numShort);
            this.Controls.Add(this.lblLong);
            this.Controls.Add(this.numLong);
            this.Name = "MockasinoSettings";
            this.Size = new System.Drawing.Size(271, 135);
            this.ResumeLayout(false);
            this.PerformLayout();

        }

        public double RSI
        {
            get
            {
                return (double)this.numRSI.Value;
            }
            set
            {
                this.numRSI.Text = value.ToString();
            }
        }

        public int MaxLong
        {
            get
            {
                return (int)this.numLong.Value;
            }
            set
            {
                this.numLong.Text = value.ToString();
            }
        }

        public int MaxShort
        {
            get
            {
                return (int)this.numShort.Value;
            }
            set
            {
                this.numShort.Text = value.ToString();
            }
        }


    }

    public class Mockasino : PosSizer, ICustomSettings
    {
        private double RSI = 40;
        private int MaxLong = 2;
        private int MaxShort = 1;

        private MockasinoSettings _settings = new MockasinoSettings();

        public override double SizePosition(Position currentPos, Bars bars, int bar, double basisPrice, PositionType pt, double riskStopLevel, double equity, double cash)
        {
            int activeLong = 0, activeShort = 0;
            const int allowedLong = 1;
            const int allowedShort = 2;      // how many longs/shorts allowed
            const double rsiThreshold = 40;      // if RSI gets above this level, act!
            const double pctEquity = 10.0;      // regular position size (% equity)
            double size = CalcPositionSize(PosSizeMode.PctEquity, pctEquity, bars, bar, pt, basisPrice, riskStopLevel, equity);

            foreach (Position p in ActivePositions)
            {
                if (p.PositionType == PositionType.Long)
                    activeLong++;
                else
                    activeShort++;
            }

            if (currentPos != null)
            {
                double rsi = (double)currentPos.Tag;

                if (rsi > rsiThreshold)
                {
                    if (currentPos.PositionType == PositionType.Long)
                    {
                        if (activeLong >= allowedLong)
                            size = 0;
                    }
                    else
                    {
                        if (activeShort >= allowedShort)
                            size = 0;
                    }
                }
            }

            return size;
        }

        #region ICustomSettings Members

        public override string GetConfigString()
        {
            return base.GetConfigString()
                + this.RSI + "|"
                + this.MaxLong + "|"
                + this.MaxShort + "|";
        }

        public override void ApplyConfigString(string config)
        {
            string[] tokens = config.Split('|');
            this.RSI = Double.Parse(tokens[0]);
            this.MaxLong = Int32.Parse(tokens[1]);
            this.MaxShort = Int32.Parse(tokens[2]);
        }

        public void ChangeSettings(UserControl ui)
        {
            this.RSI = this._settings.RSI;
            this.MaxLong = this._settings.MaxLong;
            this.MaxShort = this._settings.MaxShort;
        }

        public UserControl GetSettingsUI()
        {
            this._settings.RSI = this.RSI;
            this._settings.MaxLong = this.MaxLong;
            this._settings.MaxShort = this.MaxShort;
            return this._settings;
        }

        public void ReadSettings(ISettingsHost host)
        {
            string s = "PosSizer.Mockasino.";
            this.RSI = host.Get(s + "RSI", (double)40);
            this.MaxLong = host.Get(s + "MaxLong", (int)2);
            this.MaxShort = host.Get(s + "MaxShort", (int)1);
        }

        public void WriteSettings(ISettingsHost host)
        {
            string s = "PosSizer.Mockasino.";
            host.Set(s + "RSI", this.RSI);
            host.Set(s + "MaxLong", this.MaxLong);
            host.Set(s + "MaxShort", this.MaxShort);
        }

        #endregion

        public override string FriendlyName
        {
            get
            {
                return "Mockasino";
            }
        }
    }
}